

Tablon: Server for the Parallel Computing Course
================================================================================

* Javier Fresno 2013-2016
* Last version: 5.5
* Arturo Gonzalez-Escribano 2017-2021
* Last version: 8.0
* Daniel López Martínez 2022
* Last version: Tablon3 2.0


Dependencies
---------------------------------------------------------------------

* Pyhton3: It works with python 3.6.8.

* Python PIP: The command to install the python modules.
	- Ubuntu: apt-get install python3-pip
	- Centos: yum     install python3-pip
	
* Jinja2: This is the HTML template system.
	- Using PIP:
		pip3 install jinja2
	- If you don't like root, you can install the modules only for your user.
		pip3 install --user jinja2

* SRP: Secure Remote Password protocol ( https://github.com/cocagne/pysrp)
	SRP is a cryptographically strong authentication protocol for password-based,
	mutual authentication over an insecure network connection.
	- I have included the python version with the Tablon code but if you install
	  this library, Tablon will use the system library that is faster because
	  is implemented using C.
	- Install:
		pip3 install srp


* Pygal: Is a python module to print plots.
	- If it is not installed the stats page will not work.
	- Install:
		pip3 install pygal

* MySQL: This is the used database system
	- Ubuntu: apt-get install mysql-server
	- Centos: https://www.linode.com/docs/databases/mysql/how-to-install-mysql-on-centos-7
		sudo yum install mysql-server
		sudo yum install mysql-devel
		sudo systemctl start mysqld

	- Install python interface:
		pip3 install mysqlclient

	- Create the database (e.g. tablon1617)
		sudo mysql -u root
		show databases;
		CREATE DATABASE tablon1617;
		CREATE USER 'tablon'@'localhost' IDENTIFIED BY 'password';
		GRANT ALL PRIVILEGES ON tablon.* TO 'tablon'@'localhost';
		mysql -u tablon1617 -ppassword < db_structure.sql

Build a sandbox
---------------------------------------------------------------------

To execute student's programs safely, they are executed into
a sandbox. To build a sandbox for local queues in the server host:

1. Create a user named sandbox with home directory /home/sandbox
2. Create the directories /home/sandbox/bin and /home/sandbox/exec
3. Include in the server firewall rules to avoid processes of the
	sandbox user to start network connections
4. Add a file with the following information in the /etc/sudoers.d/ folder:

## Allows tablon to execute commands in the sandbox user account
tablon	ALL=(sandbox) NOPASSWD: <execution_program_or_script>, NOPASSWD: /usr/bin/kill

The execution program or script can be java, python, any other language
interpreter, or a simple script that executes a compiled program received
as argument. There are examples in the exec_scripts folder of tablon.


Slurm queues
---------------------------------------------------------------------

The user executing the tablon server should be in group of Slurm users that 
can execute in the selected partitions and computing nodes.

When using Slurm queues, it is not needed to create a sandbox user
on the computation nodes. The user executing the server can be created in 
the computation nodes and restrictions of that user are applied to 
the executed programs.



Configure Tablon
---------------------------------------------------------------------

All the Tablon configuration is done in the file server.py.
NOTE: Do not edit and save changes to server.py file while tablon is running.

In server.py file there is a class named CONFIG, it contains the configuration
attributes: server name, ip, port, time config, admin list...

This file also contains the code to create the queues.
Currenly, there are five types (with different python classes):
	tablon.LocalSeqQueue
		For single-thread sequential programs executed in the tablon server 
	tablon.LocalOpenMPQueue
		For multiple-thread programs executed in the tablon server 
	tablon.LocalMPIQueue
		For MPI multiple-process programs executed in the tablon server 
	tablon.LocalMPIOpenMPQueue
		For mixed MPI-OpenMP programs executed in the tablon server 
	tablon.SlurmQueue
		For OpenMP, MPI, CUDA, etc. programs executed in a cluster
		using a Slurm queue manager

There could be as many queues as needed, the arguments of the constructor are:
the name of the queue, the makefile used to compile the user code and other
parameters such as: the maximum number of threads/processes, timewall...
The queue name is used by the students to submit requests. The request
is executed in the queue and the results and other request data are shown
in the tablon web page.

Example of a OpenMP queue:

q_omp = tablon.LocalOpenMPQueue('openmp', makefile='Makefile_openmp',
	threads=16,timewall=60*2, description="OpenMP queue")

In addition to queues, Tablon has leaderboards. A leaderboard is used to rank 
the student solutions to a given problem. The leaderboards are used in the 
same way as the queues. They also have a name used by the students to send 
requests. 
A leaderboard is associated to a queue. When it receives a request, it submits 
several executions of the program to the associated queue, with preconfigured 
inputs and arguments. It checks the correctness of the solutions printed by
the program, and optionally, the accumulated execution time. Each test is a 
named a Phase of the leaderboard. 

Example of the creation of a leaderboard:

lb_cadenas = q_omp.addLeaderboard('cadenasomp',isopen=False)
lb_cadenas.addPhase({'nprocs':1,'threads':14,'arguments':'Input1 Input2 Input3'},[0,12,-1])
lb_cadenas.addPhase({'nprocs':1,'threads':14,'arguments':'BiggerInput1 BiggerInput2'},[1234,12345])

In the file server.py you can find more examples.


User codes and compilation
---------------------------------------------------------------------

The user codes are stored in the "usercodes/src" folder, the filename is the
request id. Tablon modifies the code adding a comment with the request info.
Optionally, it can add preconfigured header files. 

Tablon copies in the "usercodes/src" folder the compilation Makefiles and 
other header files defined in the queues. These files are copied each time 
the server is started.

The user codes are compiled using the Makefile of the current queue in the
folder for source files, storing the compiled programs in the "usercodes/bin"
folder.


User creation
---------------------------------------------------------------------

The users are created using the python script useradmin.py. They can be 
created one by one or by using a file with a list of user name. If this script
do not receive the password, it generates one randomly.

NOTE: Changes to the users data are promoted to the database and activated
in the next reset cycle of the tablon server.


File structure
---------------------------------------------------------------------

* client.py:
	Client program code. This python program will only work in the server.
	The users should download the client using the web interface.
	The client that is downloaded from the web is a modified version of
	this one that also includes the code of the srp library needed to
	establish the communication with the remote server.

* Makefile
	It is used to clean up the installation, backups...

* server.py
	Main server code and configuration

* run.sh
	Restarter run script

* stop.sh
	Restarter stop script

* test_codes
	Folder with test codes

* makefiles
	Folder with the queues makefiles, they are then copied to usercodes/src by the queues

* reiniciador.py
	Restarter script

* tablon
	Main source folder

* useradmin.py
	Script to create or delete users

* httpdocs
	Directory with the web static files

* README.md
	This file

* serverdata
	Folder with some server information, such as users main data, etc. 
	(Do not modify)

* ssl
	Folder with the ssl key

* templates
	HTML templates 

* usercodes
	Folder with the user codes (usercodes/src) and the binary user programs (usercodes/bin)


Reiniciador
---------------------------------------------------------------------

Tablon can be started just by calling ./server.py. But we also provide
a cleanning and restarter script. The restarter stops and runs 
again the tablon server regularly after a configurable time period.
It eliminates zombie processes, frozen conecctions, etc.

The restarter script has to be started using the ./run.sh script,
this scripts calls nohup and saves the process pid in a txt file.
The ./stop.sh script is used to stop a running server. It reads the pid,
kills the tablon server, and the restarter.

Moreover, the restarter script, checks if the server.py file has changed
to also restart the Tablon.


Reference results in the leaderboards
---------------------------------------------------------------------

The admin users (defined in a list in the server.py file) can use the
client option -r (--reference) when sending executions to a leaderboard.
Using this option, instead of showing the user name, Tablon shows "reference"
as the name and it does not include it in the ranking.


IP Tables and Proxys. Running several servers
---------------------------------------------------------------------

The configuration of tablon allows to specify the ports where the server
is listening. This allows the execution of several instances of tablon in the
same host. Note that the client should be able to reach the server port to 
submit requests. Be careful with firewall configurations.

A typical configuration for multiple instances of tablon in the same host
uses firewall redirections or a full web-proxy application to use a single 
port and redirect connections to the proper instance using for example an 
URL filter.


CSS and javascript
---------------------------------------------------------------------

Tablon uses an external css style: http://purecss.io/

Moreover, it uses several well-know javascript libraries: 
	jquery, moment, highlight. 

The .css and .js files are located in CDNs (Content distribution network).


How does Tablon work?
---------------------------------------------------------------------

Tablon uses the basic BaseHTTPServer that is included in the python 
distribution. It is an unique standalone application.

Tablon listens from a port (8080 by default) to serve both the web pages and
the execution requests. The web pages are served by normal HTTP GET requests.
The user execution requests are HTTP CONNECT requests used to send the
user source code. It uses a tailored protocol. It first authenticate the user
using the SRP library, then it serializes a python object with the user code
and the execution parameters (queue, processes, threads, ...)

The received user source code is stored in a file and a new execution request
is created in the appropriate execution queue. Tablon has an executionmanager
that keeps an execution requests queue. It compiles and stars the programs using
several executionqueue objects. A executionqueue class implements the same 
interface so the executionmanager can use it, but each class implements 
different types of queues (Sequential, OpenMP, MPI, and Slurm).


