#!/usr/bin/python3
# -*- coding: utf-8 -*- 

################################################################################
#
# Servidor Tablon v5.5
# (c) 2015, Javier Fresno
#
# Servidor Tablon v8.0
# (c) 2021, Arturo Gonzalez-Escribano
#
# Servidor Tablon3 v2.0
# (c) 2022, Daniel López Martínez
#
################################################################################


#
# Import the tablon module
#
import tablon


# For hostname
import socket

#
# This class is passed to the tablon server
#
class CONFIG:

	last_request = 0

	# Tablon title
	web_title = u'Pruebas Tablon3'

	# Just copy the hostname in the CONFIG
	hostname = socket.gethostname()

	# If the hostname is not one of the machines we are at the test host.
	if hostname not in ['tablon-xxx']:
		hostname = 'test'

	# Tablón version
	# It is important because the .js and .css files include the
	# name of the version in the url has a get parameter to
	# avoid caching when a new version is released
	version = '2.0'

	# Where to serve (by default)
	host='tablon-xxx.infor.uva.es'

	# Change the host based on the hostname (for infor.uva.es)
	if hostname != 'test':
		host =  hostname + '.infor.uva.es'

	# Main color
	color = '#40C350' # Verde prueba

	# Color & favicon For a particular machine
	if hostname == 'my_machine_name':
		color = '#1f8dd6' # Azul
		favicon = 'favicon_azul.ico'

	# Port for the HTTP server
	port=8080

	# This is the real port in case we are redirecting the connections
	real_port=8080
	
	# Port for the web pages when using it through a proxy 
	# to show several Tablon servers in the same node
	proxy_port=80

	# Port for the https server
	sport=40000

	# Prefix for multiple servers with a proxy
	web_prefix='/'

	# Optional: Links to tablon servers of previous courses in the menu
        previousCourses = [ 
                { "label" : "Spring 2021", "link" : "/2021/" },
                { "label" : "Spring 2020", "link" : "/1920/" },
                { "label" : "Spring 2019", "link" : "/1819/" },
                { "label" : "Spring 2018", "link" : "/1718/" }
                ]

	# MySQL information
	bd_host = "localhost"
	bd_user = "tablon"
	bd_pass = "xxxxxxx"
	bd_name = "tablon2021"

	# Show debug messages
	verbose=True

	# HTTP frontend number of threads
	# The frontend handles web and launch request.
	# If this is more than 1, it runs another implementation of the HTTP server
	# that uses a pool of threads to allow simultaneous user requests.
	frontend_threads = 4

	# Timeout for http connections
	http_timeout = 15
	if hostname == 'test':
		http_timeout = 60

	# Supported client version
	client_version = '1.8'

	# Send a compressed gzip auto executable python script
	# It has half the size and it is harder to hack.
	compress_client = True

	# Max size of the code
	max_program_size = 15000 * 10 # This is 10 times the size of my mpi version

	# The minimun time between user requests in seconds
	time_between_requests = 20
	if hostname == 'test':
		time_between_requests = 2

	# The minimun time between user requests in seconds
	# If case that they have used all the quota
	# (0 means no request are permitted)
	# Clases: (a) Quota exhausted, (b) more than 10% extra, (b) more than 25% extra
	# v2
	time_between_requests_no_quota = 60
	time_between_requests_no_quota_10 = 120
	time_between_requests_no_quota_25 = 240
	# v3
	#time_between_requests_no_quota = 60
	#time_between_requests_no_quota_more10 = 120
	#time_between_requests_no_quota_more25 = 180
	#time_between_requests_no_quota_more50 = 240

	# Stop receiving requests X secons before the restart
	time_before_restart = 10

	# Number of programs in the main page
	requestnumber = 20

	# Size of the stdout and stderr
	maxstdoutsize = 2500 

	# Forbidden code in the programs
	forbidden={
		"C": [
			'##',
			'fork',
			'exec',
			'popen',
			'fopen',
			'open',
			'setjmp',
			'remove',
			'rename',
			'system',
			'getenv',
			'MPI_File_open',
			'sys/syscall.h',
			'sys/stat.h',
			'fstream'
			],
		"asm": [ ],
		"sonarqube": [ ]
		}

	# Admin users: they do not have time restrictions
	# and they can send a program to the leaderboard as reference
	# This user names may not exists
	#admins = ['javi','admin1','teacher1','teacher2']
	admins = ['t1','t2','t3']

	# Enable the admin pages
	# It uses https secure connections
	# but if not used it is better to disable it
	adminpages = False

	# Users and passwords of the admin pages
	adminpass = [('admin','...')]
	if hostname == 'test':
		adminpass.append(('test','test'))

	# Expire html content, used for files such as css or js (in hours)
	expire_html = 10


#
# List of extra files to compile specific types of codes
# Using a tuple, we can change the name during the copy
#

#
# List of extra files to compile C/C++ codes
# We add them to the C/C++ queues so they are copied to the compilation folder
#
#C_extrafiles = [('cputils_2021.h','cputils.h'),'cputilstablon.h']
C_extrafiles = []

# Examples that choose different files depending on the target machine name
# Example1: 
#	Copy the cputils.h file from the makefile folder
# 	to the compilation folder. It also copies the cputilstablon_m1.h and
#	rename it to cputilstablon.h
#if CONFIG.hostname == 'machine1':
#	C_extrafiles = ['cputils.h',('cputilstablon_m1.h','cputilstablon.h')]
# Example2:
#if CONFIG.hostname == 'machine2':
#	C_extrafiles = ['cutil32.lib','libcutil.a','libcutilD.a','cutil.h','cputils.h',('cputilstablon_m2.h','cputilstablon.h')]

#
# List of extra files to compile CUDA codes
# We add them to the CUDA queues so they are copied to the compilation folder
#
#CUDA_extrafiles = [('cputils_2020.h','cputils.h'),'cputilstablon.h','int_float.h','cuda_check.h','taylor_trig.h','glibc_nrand48.h']
#CUDA_extrafiles = [('cputils_2021.h','cputils.h'),'cputilstablon.h']
CUDA_extrafiles = []

# Examples
#
# Create the queues
#

# Slurm OpenMP queue
q_omp = tablon.SlurmQueue('openmp',
	exclusive=True, 
	extensions=['.c','.cpp'], 
	forbidden_code='C',
	makefile='Makefile_openmp',
	threads=64, 
	slurm_queue='slurm_queue', 
	slurm_nodes=['slurm_node'], 
	remote_binpath="remote_binpath",
	local_binpath="local_binpath",
	timewall=50,
	concurrentjobs=1,
	description='OpenMP Queue')

# Mars queue 
q_mars = tablon.LocalSeqQueue('mars', 
	isopen=True, 
	extensions=['.asm'], 
	forbidden_code='asm',
	local_binpath="localpath_binpath",
	makefile='Makefile_mars', 
	extrafiles=['Mars4_5.jar'], 
	timewall=20, 
	concurrentjobs=6, 
	interpreter='mars.sh', 
	description='Mars Queue')

# Leaderboard OpenMP
if True:
	lb_omp = q_omp.addLeaderboard('openmplb', 
		isopen=True,

		on_error_keep_scores=True,	# Default value for phases and subphases
		shouldpass=True,	# Default value for phases and subphases
		removeblanks=False,	# Default value for phases and subphases
		description=u"Leaderboard OpenMP",

		# Strings to be searched in output to find the values of metrics (first scores)
		metric_capture=['Time'],

		# Determine if a metric is accumulated across phases, or it is registered only in the last phase
		metric_accumulate=[True],

		# Determine if a metric missed or malformed in the output generates an error or it is simply 
		# assumed it has value 0.
		metric_required=[True],

		# Computed scores: Formula of the scores showed to the users
		# Strings with arithmetic expression in terms of:
		#	(a) Metric values
		#	(b) Scores accumulated by phase triggers
		# A string can contain a single metric/trigger score name, or an expression
		# "score0", "score1", etc. represent the metrics and the trigger scores
		score_formulae=['score1', 'score0'],

		# Format used for showing scores, in printf style
		#score_format=['%.0f','%.4f'],
		score_format=['%2d','%.4f'],

		# Titles of the computed scores showed to the users
		#score_title=['Puntos','Tiempo'],
		score_title=['Puntos', 'Tiempo'],

		# Minium values of each computed score value. Values below minimum are marked in red color
		#score_minimum=[1,0],
		score_minimum=[50, 0, 0],

		# Classification criteria to sort the leaderboard.
		# Several strings representing computed scores whose values are used to sort
		# the entries and create the ranking. The first one indicates the main
		# computed score whose value is used to rank. The following ones are used to 
		# solve ties in case that the previous classification fields have the same values.
		# NOTE: "timestamp asc" is added at the end automatically, to solve complete ties
		# The first one to submit and get those scores appears higher in the ranking
		#
		# Each string contains two elements separated by space:
		#	1) The name of a computed score	denoted by sX 
		#		X in [0:number of formula minus one]
		#	2) The "asc" or "desc" keyword to represent the order
		#
		#score_classification=['s0 desc', 's1 asc']
		score_classification=['s0 desc', 's1 asc']
		)
		
# Phases
lb_omp.addPhase({'nprocs':1,'threads':48,'arguments':'1 0 2 3'}, 
				[ { 'results':'1, 0, 2, 3', 'score1':100 } ] )

if True:
	lb_fc = q_mars.addLeaderboard('lb_practica1', 
		isopen=True,
		
		shouldpass=False,       # Default value for phases
		removeblanks=True,      # Default value for phases
		description=u"Leaderboard Mars",
		
		# Strings to be searched in output to find the values of metrics (first scores)
		metric_capture=['Instructions executed','Code lines'],
		
		# Determine if a metric is accumulated across phases, or it is registered only in the last phase
		metric_accumulate=[True, False],
		
		# Formula of the scores showed to the users:
		# Types: (a) Metrics, (b) scores accumulated by phases triggers, (c) calculated scores
		# In this example, the phases use score4 to account correct results and score3 for exceptions
		#score0 --> Instrucciones
		#score1 --> Lineas
		#score2 --> excepcion
		#score3 --> Casos superados
		#score4 --> porcentage total
		#score_formulae=['score0', 'score1', 'score2', 'score3', 'score3*100/13'],
		score_formulae=['score0 - (345-2)', 'score1', 'score2', 'score3', 'score3*100/2'],
		
		# Format used for showing scores, in printf style
		score_format=['%.0f','%.0f','%.0f','%.0f','%.2f'],
		
		# Titles of the scores showed to the users
		score_title=[ 'Contador Instrucciones','L&iacute;neas C&oacute;digo','Excepc.','Casos','% superado'],
		
		# Minium values of each score value. Values below minimum are marked in red color
		score_minimum=[0,0,0,0,0],
		
		# Classification criteria to sort the leaderboards.
		# Each string contains the name of a score-column (sX) and asc/desc
		# "timestamp asc" is added at the end automatically, to solve ties
		score_classification=['s3 desc, s0 asc, s1 asc, s2 asc']
		)

# Phases
lb_fc.addPhase( {'arguments':'2#1#5#0.1'}, [
						{ 'results':'Runtime exception',        'score2':1,     'exception':True },
						{ 'results':'3.2416',                        'score3':1, }
						] )

lb_fc.addPhase( {'arguments':'0#1#10#0.2'}, [
						{ 'results':'Runtime exception',        'score2':1,     'exception':True },
						{ 'results':'ERRORINTERVALO',                   'score3':1, }
						] )

#
# Create and run the server
#
tablon.run(CONFIG)


