#!/usr/bin/python3
# -*- coding: utf-8 -*- 

################################################################################
#
# Servidor Tablon
# Asignatura Computación paralela
# Javier Fresno, 2015
#
################################################################################

import time
tstart = time.time()

#
# Logging
#
import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger('tablon')

#
# Instance to this module
#
import sys
tablon = sys.modules['tablon'] 


# Import the database
from . import tablondb

# Import the users
from .users import UserList

from .executionqueue import *


# os to use the _exit to force
import os

from . import utils


# The tablon queue list
# It also has references to queue that act as leaderboards
tablon.queues = dict()
# List with only the queues
tablon.onlyqueues = []
# List with the leaderboard names
tablon.leaderboards = []


#
# The main function to run the tablon
#
def run(config):

	# Builtin module, used to add the CONFIG object class as global
	import builtins
	builtins.CONFIG = config

	# Show all the messages
	if CONFIG.verbose:
		logger.setLevel(logging.DEBUG)
		logger.debug("Debug is enabled")

	# Start message
	logger.info("Starting Tablon")

	if len(sys.argv) == 2:
		tablon.restartat = int(sys.argv[1]) + int(utils.get_timestamp())
	else:
		tablon.restartat = None


	# Import the frontend and execution manager
	from .frontend import Frontend
	from .executionmanager import ExecutionManager



	# Load user list
	tablon.users = UserList()

	# Init the database
	tablondb.init(tablon.users,tablon.leaderboards)


	# Run the execution manager
	exemanager = ExecutionManager()
	tablon.exemanager = exemanager
	exemanager.run()

	# Run a http frontend
	frontend = Frontend()
	if not frontend.run():
		exemanager.start_stop()
		sys.exit(-1)


	logger.info("Init in %s s" % (time.time()-tstart))


	# Wait for Control+C
	try:
		while True:
			time.sleep(2)
	except:
		print()

	logger.info("Closing tablon ...")

	frontend.start_stop()
	exemanager.start_stop()

	try:
		while frontend.is_running() and exemanager.is_running():
			time.sleep(0.1)
	except:
		print("Forced")
		# Kill the process, do not wait for threads
		os._exit(-1)


	# Close the database
	tablondb.close()

	logger.info("Tablon closed")







